<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Aybat Duyshokov <duyshokov@gmail.com>
 * @package ow_plugins.links.controllers
 * @since 1.0
 */
class LINKS_CTRL_Save extends OW_ActionController
{

    public function index( $params )
    {

        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new AuthenticateException();
        }

        if ( !OW::getUser()->isAuthorized('links', 'add') )
        {
            $this->setTemplate(OW::getPluginManager()->getPlugin('base')->getCtrlViewDir() . 'authorization_failed.html');

            return;
        }

        $this->setPageHeading(OW::getLanguage()->text('links', 'save_page_heading'));
        $this->setPageHeadingIconClass('ow_ic_link');

        $plugin = OW::getPluginManager()->getPlugin('links');

        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, $plugin->getKey(), 'main_menu_item');

        $id = empty($params['id']) ? null : $params['id'];

        $service = LinkService::getInstance();

        if ( $id !== null && intval($id) > 0 )
        {
            $link = $service->findById($id);
        }
        else
        {
            /*
              @var $link Link
             */
            $link = new Link();
            $link->setUserId(OW::getUser()->getId());
        }

        $form = new LinkSaveForm($link);
        $this->addForm($form);

        if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
        {
            $form->process();
            $this->redirect(OW::getRouter()->urlForRoute('link', array('id' => $link->getId())));
        }
    }

    public function delete( $params )
    {
        $id = $params['id'];

        $service = LinkService::getInstance();

        $dto = $service->findById($id);

        $service->delete($dto);

        if ( OW::getPluginManager()->isPluginActive('activity') )
        {
            ACTIVITY_BOL_Service::getInstance()->deleteAction(null, 'link-add', $id);
        }

        BOL_FlagService::getInstance()->deleteByTypeAndEntityId('link', $id);

        $this->redirect(OW::getRouter()->urlForRoute('links'));
    }
}

class LinkSaveForm extends Form
{
    /**
     * 
     * @var Link
     */
    private $link;

    public function __construct( $link )
    {
        parent::__construct('save');

        $language = OW::getLanguage()->getInstance();

        $this->link = $link;
        $urlField = new TextField('url');

        $urlField->setHasInvitation(true)->setInvitation('http://www.example.com');

        $this->addElement(
                $urlField->setRequired(true)->
                addValidator(new UrlValidator())->
                setLabel($language->text('links', 'save_form_url_field_label'))->
                setValue($this->link->getUrl()));

        $titleField = new TextField('title');

        $this->addElement($titleField->setRequired(true)
                ->setLabel($language->text('links', 'save_form_title_field_label'))
                ->setValue($this->link->getTitle())
        );

        $descriptionTextArea = new Textarea('description');

        $this->addElement($descriptionTextArea->setRequired(true)
                ->setLabel($language->text('links', 'save_form_desc_field_label'))
                ->setValue($this->link->getDescription())
        );

        $tagService = BOL_TagService::getInstance();

        $tags = array();

        if ( intval($this->link->getId()) > 0 )
        {
            $arr = $tagService->findEntityTags($this->link->getId(), 'link');

            foreach ( (!empty($arr) ? $arr : array() ) as $dto )
            {
                $tags[] = $dto->getLabel();
            }
        }

        $tagsField = new TagsField('tags', $tags);
        $this->addElement($tagsField->setLabel($language->text('links', 'save_form_tags_field_label')));

        $submit = new Submit('submit');

        $this->addElement($submit);
    }

    public function process()
    {
        $service = LinkService::getInstance();
        $data = $this->getValues();

        $url = (mb_ereg_match('^http:\\/\\/', $data['url']) ? $data['url'] : 'http://' . $data['url']);

        $this->link->setTimestamp(time())->setUrl($url)->setDescription($data['description'])->setTitle($data['title']);

        $tags = array();

        $isNew = empty($this->link->id);

        $service->save($this->link);

        if ( intval($this->link->getId()) > 0 )
            $tags = TagsField::getTags($data['tags']);

        $tagService = BOL_TagService::getInstance();

        $tagService->updateEntityTags($this->link->getId(), 'link', $tags);

        if ( !$isNew )
        {
            return;
        }

        if ( OW::getPluginManager()->isPluginActive('activity') )
        {
            $action = new ACTIVITY_BOL_Action();
    
            $data = array(
                'string' => OW::getLanguage()->text('links', 'activity_string',
                    array(
                        'user' => BOL_UserService::getInstance()->getDisplayName($this->link->getUserId()),
                        'userUrl' => BOL_UserService::getInstance()->getUserUrl($this->link->getUserId()),
                        'title' => $this->link->getTitle(),
                        'url' => $this->link->getUrl(),
                        'permalink' => OW::getRouter()->urlForRoute('link', array('id' => $this->link->getId()))
                    )
                ),
                'content_comment' => $this->link->getDescription(),
            );
    
            $action->setType('link-add')
                ->setEntityId($this->link->id)
                ->setUserId($this->link->getUserId())
                ->setTimestamp(time())
                ->setData($data);
    
            ACTIVITY_BOL_Service::getInstance()->addAction($action);
        }
        
        //Newsfeed
        $event = new OW_Event('feed.action', array(
            'pluginKey' => 'links',
            'entityType' => 'link',
            'entityId' => $this->link->getId(),
            'userId' => $this->link->getUserId()
        ));
        OW::getEventManager()->trigger($event);
    }
}
?>
